﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Phidgets;
using Phidgets.Events;

namespace PhidgetInterfaceKit
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Phidgets.InterfaceKit ik = new Phidgets.InterfaceKit(); // The basic interface kit object that controls the actual hardware

        public MainWindow()
        {
            InitializeComponent();
            this.cbOutput0.IsEnabled = false; // We don't want the controls operable until the interface kit is plugged in
            this.cbInput0.IsEnabled = false;

            ik.Attach += new AttachEventHandler(ik_Attach);  // Invoked when the interface kit is actually plugged in (Attached)
            ik.open();
        }

        // The interfacekit is attached. Add callbacks, enable the slider and checkbox, and set it all to position 0  
        void ik_Attach(object sender, AttachEventArgs e)
        {
            this.Dispatcher.Invoke(new Action(delegate()
            {
                // If you want to look for a particular ik, do something like this. Note the 283 is a device-specific serial number 
                // if (ik.SerialNumber != 283) return;
                printMessage("ik " + ik.SerialNumber + " is attached");

                // generic phdiget callbacks
                ik.Detach += new DetachEventHandler(ik_Detach);  
                ik.Error += new ErrorEventHandler(ik_Error);

                // Interface Kit-specific calls
                ik.OutputChange += new OutputChangeEventHandler(ik_OutputChange); 
                ik.InputChange += new InputChangeEventHandler(ik_InputChange);
                ik.SensorChange += new SensorChangeEventHandler(ik_SensorChange);

                // Let the user check the checkbox
                this.cbOutput0.IsEnabled = true;
            }));
        }

        // The servo is detached. Disable various input controls and remove callbacks  
        void ik_Detach(object sender, DetachEventArgs e)
        {
            this.Dispatcher.Invoke(new Action(delegate()
            {
                printMessage("InterfaceKit Detached");
                this.cbOutput0.IsEnabled = false;
                ik.Detach -= new DetachEventHandler(ik_Detach);
                ik.Error -= new ErrorEventHandler(ik_Error);
            }));
        }

        // Some random error was generated. Rarely used but included for completeness 
        void ik_Error(object sender, ErrorEventArgs e)
        {
            this.Dispatcher.Invoke(new Action(delegate()
            {
                printMessage("InterfaceKit Error: " + e.Description);
            }));
        }

        // The phidget indicated that the state of  input #0 has changed. Make the checkbox reflect that
        void ik_InputChange(object sender, InputChangeEventArgs e)
        {
            if (e.Index != 0) return; // Ignore all but the first input
            this.Dispatcher.Invoke(new Action(delegate()
            {
                this.cbInput0.IsChecked = (ik.inputs[e.Index] == true);
            }));
        }

        // An output to a phidget has changed. Display that as feedback
        void ik_OutputChange(object sender, OutputChangeEventArgs e)
        {
             this.Dispatcher.Invoke(new Action(delegate()
            {
                printMessage("Output #" + e.Index.ToString() + " changed to " + e.Value.ToString());
            }));
        }

        // A phidget's sensor value (which ranges between 0-1000) has changed
        void ik_SensorChange(object sender, SensorChangeEventArgs e)
        {
            if (e.Index != 0) return; // Ignore all but the first sensor
            this.Dispatcher.Invoke(new Action(delegate()
            {
                this.tbSensor0.Text = ik.sensors[e.Index].Value.ToString(); ;
            }));
        }

        // THe user checked the output checkbox - power it up
        private void cbOutput0_Checked(object sender, RoutedEventArgs e)
        {
             ik.outputs[0] = true;
        }


        // THe user unchecked the output checkbox - power it down
        private void cbOutput0_Unchecked(object sender, RoutedEventArgs e)
        {
            ik.outputs[0] = false;
        }
        
        // Put some feedback in the title bar
        private void printMessage(string msg)
        {
            this.Title = msg;
        }
    }
}
